#ifndef	_notify_server_
#define	_notify_server_

/* Module notify */

#include <mach/kern_return.h>
#if	(defined(__STDC__) || defined(c_plusplus)) || defined(LINTLIBRARY)
#include <mach/port.h>
#include <mach/message.h>
#endif

#include <mach/std_types.h>

/* SimpleRoutine mach_notify_port_deleted */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t do_seqnos_mach_notify_port_deleted
#if	defined(LINTLIBRARY)
    (notify, seqno, name)
	mach_port_t notify;
	mach_port_seqno_t seqno;
	mach_port_t name;
{ return do_seqnos_mach_notify_port_deleted(notify, seqno, name); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	mach_port_t notify,
	mach_port_seqno_t seqno,
	mach_port_t name
);
#else
    ();
#endif
#endif

/* SimpleRoutine mach_notify_msg_accepted */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t do_seqnos_mach_notify_msg_accepted
#if	defined(LINTLIBRARY)
    (notify, seqno, name)
	mach_port_t notify;
	mach_port_seqno_t seqno;
	mach_port_t name;
{ return do_seqnos_mach_notify_msg_accepted(notify, seqno, name); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	mach_port_t notify,
	mach_port_seqno_t seqno,
	mach_port_t name
);
#else
    ();
#endif
#endif

/* SimpleRoutine mach_notify_port_destroyed */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t do_seqnos_mach_notify_port_destroyed
#if	defined(LINTLIBRARY)
    (notify, seqno, rights)
	mach_port_t notify;
	mach_port_seqno_t seqno;
	mach_port_t rights;
{ return do_seqnos_mach_notify_port_destroyed(notify, seqno, rights); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	mach_port_t notify,
	mach_port_seqno_t seqno,
	mach_port_t rights
);
#else
    ();
#endif
#endif

/* SimpleRoutine mach_notify_no_senders */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t do_seqnos_mach_notify_no_senders
#if	defined(LINTLIBRARY)
    (notify, seqno, mscount)
	mach_port_t notify;
	mach_port_seqno_t seqno;
	mach_port_mscount_t mscount;
{ return do_seqnos_mach_notify_no_senders(notify, seqno, mscount); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	mach_port_t notify,
	mach_port_seqno_t seqno,
	mach_port_mscount_t mscount
);
#else
    ();
#endif
#endif

/* SimpleRoutine mach_notify_send_once */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t do_seqnos_mach_notify_send_once
#if	defined(LINTLIBRARY)
    (notify, seqno)
	mach_port_t notify;
	mach_port_seqno_t seqno;
{ return do_seqnos_mach_notify_send_once(notify, seqno); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	mach_port_t notify,
	mach_port_seqno_t seqno
);
#else
    ();
#endif
#endif

/* SimpleRoutine mach_notify_dead_name */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t do_seqnos_mach_notify_dead_name
#if	defined(LINTLIBRARY)
    (notify, seqno, name)
	mach_port_t notify;
	mach_port_seqno_t seqno;
	mach_port_t name;
{ return do_seqnos_mach_notify_dead_name(notify, seqno, name); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	mach_port_t notify,
	mach_port_seqno_t seqno,
	mach_port_t name
);
#else
    ();
#endif
#endif

#endif	_notify_server_
