# define sySkip 257
# define syRoutine 258
# define sySimpleRoutine 259
# define sySimpleProcedure 260
# define syProcedure 261
# define syFunction 262
# define sySubsystem 263
# define syKernelUser 264
# define syKernelServer 265
# define syMsgOption 266
# define syMsgSeqno 267
# define syWaitTime 268
# define syNoWaitTime 269
# define syErrorProc 270
# define syServerPrefix 271
# define syUserPrefix 272
# define syServerDemux 273
# define syRCSId 274
# define syImport 275
# define syUImport 276
# define sySImport 277
# define syIn 278
# define syOut 279
# define syInOut 280
# define syRequestPort 281
# define syReplyPort 282
# define sySReplyPort 283
# define syUReplyPort 284
# define syType 285
# define syArray 286
# define syStruct 287
# define syOf 288
# define syInTran 289
# define syOutTran 290
# define syDestructor 291
# define syCType 292
# define syCUserType 293
# define syCServerType 294
# define syCString 295
# define syColon 296
# define sySemi 297
# define syComma 298
# define syPlus 299
# define syMinus 300
# define syStar 301
# define syDiv 302
# define syLParen 303
# define syRParen 304
# define syEqual 305
# define syCaret 306
# define syTilde 307
# define syLAngle 308
# define syRAngle 309
# define syLBrack 310
# define syRBrack 311
# define syBar 312
# define syError 313
# define syNumber 314
# define sySymbolicType 315
# define syIdentifier 316
# define syString 317
# define syQString 318
# define syFileName 319
# define syIPCFlag 320

typedef union 
{
    u_int number;
    identifier_t identifier;
    string_t string;
    statement_kind_t statement_kind;
    ipc_type_t *type;
    struct
    {
	u_int innumber;		/* msgt_name value, when sending */
	string_t instr;
	u_int outnumber;	/* msgt_name value, when receiving */
	string_t outstr;
	u_int size;		/* 0 means there is no default size */
    } symtype;
    routine_t *routine;
    arg_kind_t direction;
    argument_t *argument;
    ipc_flags_t flag;
} YYSTYPE;
extern YYSTYPE yylval;
