#ifndef	_boot_user_
#define	_boot_user_

/* Module boot */

#include <mach/kern_return.h>
#if	(defined(__STDC__) || defined(c_plusplus)) || defined(LINTLIBRARY)
#include <mach/port.h>
#include <mach/message.h>
#endif

#include <mach/std_types.h>
#include <mach/mach_types.h>
#include <device/device_types.h>
#include <device/net_status.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <hurd/hurd_types.h>

/* Routine boot_init */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t boot_init
#if	defined(LINTLIBRARY)
    (server, startup_port, host_priv, device_master)
	mach_port_t server;
	mach_port_t startup_port;
	mach_port_t *host_priv;
	mach_port_t *device_master;
{ return boot_init(server, startup_port, host_priv, device_master); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	mach_port_t server,
	mach_port_t startup_port,
	mach_port_t *host_priv,
	mach_port_t *device_master
);
#else
    ();
#endif
#endif

#endif	_boot_user_
