.dep_alix_sysent.c alix_sysent.o : alix_sysent.c mach.h \
  /usr1/mk/release/i386_mach/latest/include/mach/mach_types.h \
  /usr1/mk/release/i386_mach/latest/include/mach/host_info.h \
  /usr1/mk/release/i386_mach/latest/include/mach/machine.h \
  /usr1/mk/release/i386_mach/latest/include/mach/machine/vm_types.h \
  /usr1/mk/release/i386_mach/latest/include/mach/boolean.h \
  /usr1/mk/release/i386_mach/latest/include/mach/machine/boolean.h \
  /usr1/mk/release/i386_mach/latest/include/mach/memory_object.h \
  /usr1/mk/release/i386_mach/latest/include/mach/port.h \
  /usr1/mk/release/i386_mach/latest/include/mach/processor_info.h \
  /usr1/mk/release/i386_mach/latest/include/mach/task_info.h \
  /usr1/mk/release/i386_mach/latest/include/mach/time_value.h \
  /usr1/mk/release/i386_mach/latest/include/mach/task_special_ports.h \
  /usr1/mk/release/i386_mach/latest/include/mach/thread_info.h \
  /usr1/mk/release/i386_mach/latest/include/mach/policy.h \
  /usr1/mk/release/i386_mach/latest/include/mach/thread_special_ports.h \
  /usr1/mk/release/i386_mach/latest/include/mach/thread_status.h \
  /usr1/mk/release/i386_mach/latest/include/mach/machine/thread_status.h \
  /usr1/mk/release/i386_mach/latest/include/i386/fp_reg.h \
  /usr1/mk/release/i386_mach/latest/include/mach/vm_attributes.h \
  /usr1/mk/release/i386_mach/latest/include/mach/vm_inherit.h \
  /usr1/mk/release/i386_mach/latest/include/mach/vm_prot.h \
  /usr1/mk/release/i386_mach/latest/include/mach/vm_statistics.h \
  /usr1/mk/release/i386_mach/latest/include/mach/std_types.h \
  /usr1/mk/release/i386_mach/latest/include/mach/kern_return.h \
  /usr1/mk/release/i386_mach/latest/include/mach/machine/kern_return.h \
  mach_htg.h /usr1/mk/release/i386_mach/latest/include/mach/message.h \
  /usr1/mk/release/i386_mach/latest/include/mach/mach_port.h \
  /usr1/mk/release/i386_mach/latest/include/mach_init.h \
  /usr1/mk/release/i386_mach/latest/include/cthreads.h \
  /usr1/mk/release/i386_mach/latest/include/machine/cthreads.h \
  /usr1/mk/release/i386_mach/latest/include/mach/mach.h \
  /usr1/mk/release/i386_mach/latest/include/mach/machine/vm_param.h \
  /usr1/mk/release/i386_mach/latest/include/setjmp.h ../gnu/posix1_limits.h \
  ../gnu/signal.h ../machine/sigcodes.h ../hurd/hurd_types.h ../gnu/types.h \
  ../gnu/param.h ../gnu/resource.h \
  /usr1/mk/release/i386_mach/latest/include/strings.h alix_sysent.h alix.h 
