#ifndef	_fsys
#define	_fsys

/* Module fsys */

#include <mach/kern_return.h>
#if	(defined(__STDC__) || defined(c_plusplus)) || defined(LINTLIBRARY)
#include <mach/port.h>
#include <mach/message.h>
#endif

#ifdef	mig_external
mig_external
#else
extern
#endif
void init_fsys
#if	(defined(__STDC__) || defined(c_plusplus))
    (mach_port_t rep_port);
#else
    ();
#endif
#include <mach/std_types.h>
#include <mach/mach_types.h>
#include <device/device_types.h>
#include <device/net_status.h>
#include <gnu/types.h>
#include <hurd/hurd_types.h>

/* Routine fsys_startup */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t fsys_startup
#if	defined(LINTLIBRARY)
    (fsys, realnode, dotdot_dir)
	fsys_t fsys;
	file_t realnode;
	file_t dotdot_dir;
{ return fsys_startup(fsys, realnode, dotdot_dir); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	fsys_t fsys,
	file_t realnode,
	file_t dotdot_dir
);
#else
    ();
#endif
#endif

/* Routine fsys_goaway */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t fsys_goaway
#if	defined(LINTLIBRARY)
    (fsys, flags)
	fsys_t fsys;
	int flags;
{ return fsys_goaway(fsys, flags); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	fsys_t fsys,
	int flags
);
#else
    ();
#endif
#endif

/* Routine fsys_getroot */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t fsys_getroot
#if	defined(LINTLIBRARY)
    (fsys, id, idCnt, file)
	fsys_t fsys;
	idblock_t id;
	mach_msg_type_number_t idCnt;
	file_t *file;
{ return fsys_getroot(fsys, id, idCnt, file); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	fsys_t fsys,
	idblock_t id,
	mach_msg_type_number_t idCnt,
	file_t *file
);
#else
    ();
#endif
#endif

#endif	_fsys
