#define EPERM 1
#define ENOENT 2
#define ESRCH 3
#define EINTR 4
#define EIO 5
#define ENXIO 6
#define ENOEXEC 7
#define EBADF 8
#define ECHILD 9
#define EAGAIN 10
#define ENOMEM 11
#define EACCES 12
#define EFAULT 13
#define ENOTBLK 14
#define EBUSY 15
#define EEXIST 16
#define EXDEV 17
#define ENODEV 18
#define ENOTDIR 19
#define EISDIR 20
#define EINVAL 21
#define ENFILE 22
#define EMFILE 23
#define ENOTTY 24
#define ETXTBSY 25
#define EFBIG 26
#define ENOSPC 27
#define ESPIPE 28
#define EROFS 29
#define EMLINK 30
#define EPIPE 31
#define EDOM 32
#define ERANGE 33
#define EWOULDBLOCK 34
#define EINPROGRESS 35
#define EALREADY 36
#define ENOTSOCK 37
#define EDESTADDRREQ 38
#define EMSGSIZE 39
#define EPROTOTYPE 40
#define ENOPROTOOPT 41
#define EPROTONOSUPPORT 42
#define ESOCKTNOSUPPORT 43
#define EOPNOTSUPP 44
#define EPFNOSUPPORT 45
#define EAFNOSUPPORT 46
#define EADDRINUSE 47
#define EADDRNOTAVAIL 48
#define ENETDOWN 49
#define ENETUNREACH 50
#define ENETRESET 51
#define ECONNABORTED 52
#define ECONNRESET 53
#define ENOBUFS 54
#define EISCONN 55
#define ENOTCONN 56
#define ESHUTDOWN 57
#define ETOOMANYREFS 58
#define ETIMEDOUT 59
#define ECONNREFUSED 60
#define ELOOP 61
#define ENAMETOOLONG 62
#define EHOSTDOWN 63
#define EHOSTUNREACH 64
#define ENOTEMPTY 65
#define EPROCLIM 66
#define EUSERS 67
#define EDQUOT 68
#define ESTALE 69
#define EREMOTE 70
#define ENOMSG 71
#define EBADMSG 72
#define EIDRM 73
#define EDEADLK 74
#define ENOLCK 75
