#ifndef	_auth_reply
#define	_auth_reply

/* Module auth_reply */

#include <mach/kern_return.h>
#if	(defined(__STDC__) || defined(c_plusplus)) || defined(LINTLIBRARY)
#include <mach/port.h>
#include <mach/message.h>
#endif

#ifdef	mig_external
mig_external
#else
extern
#endif
void init_auth_reply
#if	(defined(__STDC__) || defined(c_plusplus))
    (mach_port_t rep_port);
#else
    ();
#endif
#include <mach/std_types.h>
#include <mach/mach_types.h>
#include <device/device_types.h>
#include <device/net_status.h>
#include <gnu/types.h>
#include <hurd/hurd_types.h>
#include <lib/libhurd.h>

/* SimpleRoutine auth_user_authenticate_reply */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t auth_user_authenticate_reply
#if	defined(LINTLIBRARY)
    (reply_port, reply_portPoly, return_code, newhandle)
	mach_port_t reply_port;
	mach_msg_type_name_t reply_portPoly;
	kern_return_t return_code;
	mach_port_t newhandle;
{ return auth_user_authenticate_reply(reply_port, reply_portPoly, return_code, newhandle); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	mach_port_t reply_port,
	mach_msg_type_name_t reply_portPoly,
	kern_return_t return_code,
	mach_port_t newhandle
);
#else
    ();
#endif
#endif

/* SimpleRoutine auth_server_authenticate_reply */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t auth_server_authenticate_reply
#if	defined(LINTLIBRARY)
    (reply_port, reply_portPoly, return_code, idblock)
	mach_port_t reply_port;
	mach_msg_type_name_t reply_portPoly;
	kern_return_t return_code;
	idblock_t idblock;
{ return auth_server_authenticate_reply(reply_port, reply_portPoly, return_code, idblock); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	mach_port_t reply_port,
	mach_msg_type_name_t reply_portPoly,
	kern_return_t return_code,
	idblock_t idblock
);
#else
    ();
#endif
#endif

#endif	_auth_reply
