

/* This file is automatically generated from
   syscalls.list by alix_makesysent. */
/* Info for each syscall */
struct sysent
{
  char *name;			/* printable name */
  void *vector;			/* address of vector routine */
  int nargs;	       		/* number of arguments */
  int modregs;			/* syscall needs 4th regs arg */
};

/* Info for a set of syscalls */
struct systab
{
  int min;			/* first entry */
  int max;			/* last entry */
  struct sysent *tab;		/* table of entries */
};

struct systab systab[];

typedef int syscall_t(void *, int *, int *);
typedef int syscallreg_t(void *, int *, int *, int *);


/* BSD syscalls */
syscall_t exit;
syscallreg_t fork;
syscall_t read;
syscall_t write;
syscall_t open;
syscall_t close;
syscall_t wait4;
#ifdef COMPAT_43
syscall_t creat_compat43;
#endif
syscall_t link;
syscall_t unlink;
syscall_t chdir;
syscall_t fchdir;
syscall_t mknod;
syscall_t chmod;
syscall_t chown;
syscall_t brk;
syscall_t getfsstat;
syscall_t lseek;
syscall_t getpid;
syscall_t mount;
syscall_t unmount;
syscall_t setuid;
syscall_t getuid;
syscall_t geteuid;
syscall_t ptrace;
syscall_t recvmsg;
syscall_t sendmsg;
syscall_t recvfrom;
syscall_t accept;
syscall_t getpeername;
syscall_t getsockname;
syscall_t access;
syscall_t chflags;
syscall_t fchflags;
syscall_t sync;
syscall_t kill;
#ifdef COMPAT_44
syscall_t stat_compat44;
#endif
syscall_t getppid;
#ifdef COMPAT_44
syscall_t lstat_compat44;
#endif
syscall_t dup;
syscall_t pipe;
syscall_t getegid;
syscall_t profil;
syscall_t sigaction;
syscall_t getgid;
syscall_t sigprocmask;
syscall_t getlogin;
syscall_t setlogin;
syscall_t acct;
syscall_t sigpending;
syscall_t sigaltstack;
syscall_t ioctl;
syscall_t reboot;
syscall_t revoke;
syscall_t symlink;
syscall_t readlink;
syscall_t execve;
syscall_t umask;
syscall_t chroot;
#ifdef COMPAT_44
syscall_t fstat_compat44;
#endif
syscall_t getkerninfo;
syscall_t getpagesize;
syscall_t msync;
#ifdef COMPAT_44
syscall_t vfork_compat44;
#endif
syscall_t sbrk;
syscall_t sstk;
syscall_t mmap;
syscall_t vadvise;
syscall_t munmap;
syscall_t mprotect;
syscall_t madvise;
syscall_t mincore;
syscall_t getgroups;
syscall_t setgroups;
syscall_t getpgrp;
syscall_t setpgid;
syscall_t setitimer;
#ifdef COMPAT_43
syscallreg_t wait_compat43;
#endif
syscall_t swapon;
syscall_t getitimer;
syscall_t gethostname;
syscall_t sethostname;
syscall_t getdtablesize;
syscall_t dup2;
syscall_t fcntl;
syscall_t select;
syscall_t fsync;
syscall_t setpriority;
syscall_t socket;
syscall_t connect;
#ifdef COMPAT_43
syscall_t accept_compat43;
#endif
syscall_t getpriority;
#ifdef COMPAT_43
syscall_t send_compat43;
#endif
#ifdef COMPAT_43
syscall_t recv_compat43;
#endif
syscallreg_t sigreturn;
syscall_t bind;
syscall_t setsockopt;
syscall_t listen;
#ifdef COMPAT_43
syscall_t sigvec_compat43;
#endif
#ifdef COMPAT_43
syscall_t sigblock_compat43;
#endif
#ifdef COMPAT_43
syscall_t sigsetmask_compat43;
#endif
syscall_t sigsuspend;
syscall_t sigstack;
#ifdef COMPAT_43
syscall_t recvmsg_compat43;
#endif
#ifdef COMPAT_43
syscall_t sendmsg_compat43;
#endif
syscall_t gettimeofday;
syscall_t getrusage;
syscall_t getsockopt;
syscall_t readv;
syscall_t writev;
syscall_t settimeofday;
syscall_t fchown;
syscall_t fchmod;
#ifdef COMPAT_43
syscall_t recvfrom_compat43;
#endif
#ifdef COMPAT_43
syscall_t setreuid_compat43;
#endif
#ifdef COMPAT_43
syscall_t setregid_compat43;
#endif
syscall_t rename;
syscall_t truncate;
syscall_t ftruncate;
syscall_t flock;
syscall_t mkfifo;
syscall_t sendto;
syscall_t shutdown;
syscall_t socketpair;
syscall_t mkdir;
syscall_t rmdir;
syscall_t utimes;
syscall_t adjtime;
#ifdef COMPAT_43
syscall_t getpeername_compat43;
#endif
syscall_t gethostid;
syscall_t sethostid;
syscall_t getrlimit;
syscall_t setrlimit;
#ifdef COMPAT_43
syscall_t killpg_compat43;
#endif
syscall_t setsid;
#ifdef COMPAT_43
syscall_t getsockname_compat43;
#endif


/* NFS syscalls */
syscall_t getdirentries;
syscall_t statfs;
syscall_t fstatfs;


/* CMU syscalls */
syscall_t task2pid;
syscall_t pid2task;


/* GNU syscalls */
syscall_t stat;
syscall_t lstat;
syscall_t fstat;
syscall_t getproc;
syscall_t setproc;
syscall_t getauth;
syscall_t setauth;
syscall_t setdtablesize;
syscall_t getdport;
syscall_t portopen;
syscall_t fchcore;
syscall_t chcore;
syscall_t setcwdir;
syscall_t setcrdir;
syscall_t setccdir;
syscall_t getcwdir;
syscall_t getcrdir;
syscall_t getccdir;
syscall_t fchroot;
