#ifndef	_name
#define	_name

/* Module name */

#include <mach/kern_return.h>
#if	(defined(__STDC__) || defined(c_plusplus)) || defined(LINTLIBRARY)
#include <mach/port.h>
#include <mach/message.h>
#endif

#ifdef	mig_external
mig_external
#else
extern
#endif
void init_name
#if	(defined(__STDC__) || defined(c_plusplus))
    (mach_port_t rep_port);
#else
    ();
#endif
#include <mach/std_types.h>
#include <mach/mach_types.h>
#include <device/device_types.h>
#include <device/net_status.h>
#include <gnu/types.h>
#include <hurd/hurd_types.h>
#include <lib/libhurd.h>

/* Routine name_getport */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t name_getport
#if	defined(LINTLIBRARY)
    (nameserver, name, nameCnt, port)
	mach_port_t nameserver;
	name_t name;
	mach_msg_type_number_t nameCnt;
	mach_port_t *port;
{ return name_getport(nameserver, name, nameCnt, port); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	mach_port_t nameserver,
	name_t name,
	mach_msg_type_number_t nameCnt,
	mach_port_t *port
);
#else
    ();
#endif
#endif

/* Routine name_registerport */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t name_registerport
#if	defined(LINTLIBRARY)
    (nameserver, name, nameCnt, port)
	mach_port_t nameserver;
	name_t name;
	mach_msg_type_number_t nameCnt;
	mach_port_t port;
{ return name_registerport(nameserver, name, nameCnt, port); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	mach_port_t nameserver,
	name_t name,
	mach_msg_type_number_t nameCnt,
	mach_port_t port
);
#else
    ();
#endif
#endif

#endif	_name
