#ifndef	_process_reply_user_
#define	_process_reply_user_

/* Module process_reply */

#include <mach/kern_return.h>
#if	(defined(__STDC__) || defined(c_plusplus)) || defined(LINTLIBRARY)
#include <mach/port.h>
#include <mach/message.h>
#endif

#include <mach/std_types.h>
#include <mach/mach_types.h>
#include <device/device_types.h>
#include <device/net_status.h>
#include <gnu/types.h>
#include <hurd/hurd_types.h>
#include <lib/libhurd.h>

/* SimpleRoutine proc_wait_reply */
#ifdef	mig_external
mig_external
#else
extern
#endif
kern_return_t proc_wait_reply
#if	defined(LINTLIBRARY)
    (reply_port, reply_portPoly, retcode, status, rusage, pid_status)
	mach_port_t reply_port;
	mach_msg_type_name_t reply_portPoly;
	kern_return_t retcode;
	int status;
	rusage_t rusage;
	pid_t pid_status;
{ return proc_wait_reply(reply_port, reply_portPoly, retcode, status, rusage, pid_status); }
#else
#if	(defined(__STDC__) || defined(c_plusplus))
(
	mach_port_t reply_port,
	mach_msg_type_name_t reply_portPoly,
	kern_return_t retcode,
	int status,
	rusage_t rusage,
	pid_t pid_status
);
#else
    ();
#endif
#endif

#endif	_process_reply_user_
