/* Generate offsets for assembler input
   Copyright (C) 1991 Free Software Foundation

This file is part of the GNU Hurd.

The GNU Hurd is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 1, or (at your option)
any later version.

The GNU Hurd is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GNU Hurd; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* Written by Michael I. Bushnell.  */

#include "mach.h"
#include <gnu/signal.h>
#include <i386/alix_machdep.h>

void
main()
{
  struct stackbase *sb = 0;
  void printf (), exit ();	/* XXX */
  
  printf ("#define UCOPY_PC %d\n", (int) &sb->ucopy_pc);
  printf ("#define UCOPY_BASE %d\n", (int) &sb->ucopy_ubase);
  printf ("#define UCOPY_LEN %d\n", (int) &sb->ucopy_len);
  printf ("#define UCOPY_SP %d\n", (int) &sb->ucopy_sp);
  printf ("#define IN_UCOPY %d\n", (int) &sb->in_ucopy);
  printf ("#define STACK_MASK %d\n", STACK_MASK);
  printf ("#define INT_STACK_SIZE %d\n", INT_STACK_SIZE);
  printf ("#define SIGSYS %d\n", SIGSYS);
  exit (0);
}

