

/* This file is automatically generated from
   syscalls.list by alix_makesysent. */
#include "mach.h"
#include <cthreads.h>
#include <gnu/posix1_limits.h>
#include <gnu/signal.h>
#include <hurd/hurd_types.h>
#include "alix_sysent.h"
#include "alix.h"
#if defined (COMPAT_43) && ! defined (COMPAT_44)
#error Cannot define COMPAT_43 without COMPAT_44
#endif
struct sysent sysent_BSD[] = 
{
  {"exit", exit, 1, 0},			/* 1  */
  {"fork", fork, 2, 1},			/* 2  */
  {"read", read, 3, 0},			/* 3  */
  {"write", write, 3, 0},		/* 4  */
  {"open", open, 3, 0},			/* 5  */
  {"close", close, 1, 0},		/* 6  */
  {"wait4", wait4, 4, 0},		/* 7  */
#ifdef COMPAT_43
  {"creat_compat43", creat_compat43, 2, 0}, /* 8  */
#else
  {"nosys", 0, 0, 0},
#endif
  {"link", link, 2, 0},			/* 9  */
  {"unlink", unlink, 1, 0},		/* 10  */
  {"nosys", 0, 0, 0},			/* 11 execv  */
  {"chdir", chdir, 1, 0},		/* 12  */
  {"fchdir", fchdir, 1, 0},		/* 13  */
  {"mknod", mknod, 3, 0},		/* 14  */
  {"chmod", chmod, 2, 0},		/* 15  */
  {"chown", chown, 3, 0},		/* 16  */
  {"brk", brk, 1, 0},			/* 17  */
  {"getfsstat", getfsstat, 3, 0},	/* 18  */
  {"lseek", lseek, 3, 0},		/* 19  */
  {"getpid", getpid, 0, 0},		/* 20  */
  {"mount", mount, 3, 0},		/* 21  */
  {"unmount", unmount, 1, 0},		/* 22  */
  {"setuid", setuid, 1, 0},		/* 23  */
  {"getuid", getuid, 0, 0},		/* 24  */
  {"geteuid", geteuid, 0, 0},		/* 25  */
  {"ptrace", ptrace, 4, 0},		/* 26 icky */
  {"recvmsg", recvmsg, 3, 0},		/* 27  */
  {"sendmsg", sendmsg, 3, 0},		/* 28  */
  {"recvfrom", recvfrom, 6, 0},		/* 29  */
  {"accept", accept, 3, 0},		/* 30  */
  {"getpeername", getpeername, 3, 0},	/* 31  */
  {"getsockname", getsockname, 3, 0},	/* 32  */
  {"access", access, 2, 0},		/* 33  */
  {"chflags", chflags, 2, 0},		/* 34  */
  {"fchflags", fchflags, 2, 0},		/* 35  */
  {"sync", sync, 0, 0},			/* 36  */
  {"kill", kill, 2, 0},			/* 37  */
#ifdef COMPAT_44
  {"stat_compat44", stat_compat44, 2, 0}, /* 38 BSD */
#else
  {"nosys", 0, 0, 0},
#endif
  {"getppid", getppid, 0, 0},		/* 39  */
#ifdef COMPAT_44
  {"lstat_compat44", lstat_compat44, 2, 0}, /* 40 BSD */
#else
  {"nosys", 0, 0, 0},
#endif
  {"dup", dup, 1, 0},			/* 41  */
  {"pipe", pipe, 0, 0},			/* 42  */
  {"getegid", getegid, 0, 0},		/* 43  */
  {"profil", profil, 4, 0},		/* 44  */
  {"nosys", 0, 0, 0},			/* 45 ktrace  */
  {"sigaction", sigaction, 3, 0},	/* 46  */
  {"getgid", getgid, 0, 0},		/* 47  */
  {"sigprocmask", sigprocmask, 2, 0},	/* 48  */
  {"getlogin", getlogin, 2, 0},		/* 49  */
  {"setlogin", setlogin, 1, 0},		/* 50  */
  {"acct", acct, 1, 0},			/* 51  */
  {"sigpending", sigpending, 0, 0},	/* 52  */
  {"sigaltstack", sigaltstack, 3, 0},	/* 53  */
  {"ioctl", ioctl, 3, 0},		/* 54  */
  {"reboot", reboot, 1, 0},		/* 55  */
  {"revoke", revoke, 1, 0},		/* 56  */
  {"symlink", symlink, 2, 0},		/* 57  */
  {"readlink", readlink, 3, 0},		/* 58  */
  {"execve", execve, 3, 0},		/* 59  */
  {"umask", umask, 1, 0},		/* 60  */
  {"chroot", chroot, 1, 0},		/* 61  */
#ifdef COMPAT_44
  {"fstat_compat44", fstat_compat44, 2, 0}, /* 62 BSD */
#else
  {"nosys", 0, 0, 0},
#endif
  {"getkerninfo", getkerninfo, 4, 0},	/* 63  */
  {"getpagesize", getpagesize, 0, 0},	/* 64  */
  {"msync", msync, 2, 0},		/* 65  */
#ifdef COMPAT_44
  {"vfork_compat44", vfork_compat44, 0, 0}, /* 66  */
#else
  {"nosys", 0, 0, 0},
#endif
  {"nosys", 0, 0, 0},			/* 67 vread  */
  {"nosys", 0, 0, 0},			/* 68 vwrite  */
  {"sbrk", sbrk, 1, 0},			/* 69  */
  {"sstk", sstk, 1, 0},			/* 70  */
  {"mmap", mmap, 6, 0},			/* 71  */
  {"vadvise", vadvise, 1, 0},		/* 72  */
  {"munmap", munmap, 2, 0},		/* 73  */
  {"mprotect", mprotect, 3, 0},		/* 74  */
  {"madvise", madvise, 3, 0},		/* 75  */
  {"nosys", 0, 0, 0},			/* 76 vhangup  */
  {"nosys", 0, 0, 0},			/* 77 vlimit  */
  {"mincore", mincore, 3, 0},		/* 78  */
  {"getgroups", getgroups, 2, 0},	/* 79  */
  {"setgroups", setgroups, 2, 0},	/* 80  */
  {"getpgrp", getpgrp, 1, 0},		/* 81  */
  {"setpgid", setpgid, 2, 0},		/* 82  */
  {"setitimer", setitimer, 3, 0},	/* 83  */
#ifdef COMPAT_43
  {"wait_compat43", wait_compat43, 0, 1}, /* 84  */
#else
  {"nosys", 0, 0, 0},
#endif
  {"swapon", swapon, 2, 0},		/* 85  */
  {"getitimer", getitimer, 2, 0},	/* 86  */
  {"gethostname", gethostname, 2, 0},	/* 87  */
  {"sethostname", sethostname, 2, 0},	/* 88  */
  {"getdtablesize", getdtablesize, 0, 0}, /* 89  */
  {"dup2", dup2, 2, 0},			/* 90  */
  {"nosys", 0, 0, 0},			/* 91 getdopt  */
  {"fcntl", fcntl, 3, 0},		/* 92  */
  {"select", select, 5, 0},		/* 93  */
  {"nosys", 0, 0, 0},			/* 94 setdopt  */
  {"fsync", fsync, 1, 0},		/* 95  */
  {"setpriority", setpriority, 3, 0},	/* 96  */
  {"socket", socket, 3, 0},		/* 97  */
  {"connect", connect, 3, 0},		/* 98  */
#ifdef COMPAT_43
  {"accept_compat43", accept_compat43, 1, 0}, /* 99  */
#else
  {"nosys", 0, 0, 0},
#endif
  {"getpriority", getpriority, 2, 0},	/* 100  */
#ifdef COMPAT_43
  {"send_compat43", send_compat43, 4, 0}, /* 101  */
#else
  {"nosys", 0, 0, 0},
#endif
#ifdef COMPAT_43
  {"recv_compat43", recv_compat43, 4, 0}, /* 102  */
#else
  {"nosys", 0, 0, 0},
#endif
  {"sigreturn", sigreturn, 1, 1},	/* 103  */
  {"bind", bind, 3, 0},			/* 104  */
  {"setsockopt", setsockopt, 5, 0},	/* 105  */
  {"listen", listen, 2, 0},		/* 106  */
  {"nosys", 0, 0, 0},			/* 107 vtimes  */
#ifdef COMPAT_43
  {"sigvec_compat43", sigvec_compat43, 3, 0}, /* 108  */
#else
  {"nosys", 0, 0, 0},
#endif
#ifdef COMPAT_43
  {"sigblock_compat43", sigblock_compat43, 1, 0}, /* 109  */
#else
  {"nosys", 0, 0, 0},
#endif
#ifdef COMPAT_43
  {"sigsetmask_compat43", sigsetmask_compat43, 1, 0}, /* 110  */
#else
  {"nosys", 0, 0, 0},
#endif
  {"sigsuspend", sigsuspend, 1, 0},	/* 111  */
  {"sigstack", sigstack, 2, 0},		/* 112  */
#ifdef COMPAT_43
  {"recvmsg_compat43", recvmsg_compat43, 3, 0}, /* 113  */
#else
  {"nosys", 0, 0, 0},
#endif
#ifdef COMPAT_43
  {"sendmsg_compat43", sendmsg_compat43, 3, 0}, /* 114  */
#else
  {"nosys", 0, 0, 0},
#endif
  {"nosys", 0, 0, 0},			/* 115 vtrace  */
  {"gettimeofday", gettimeofday, 2, 0},	/* 116  */
  {"getrusage", getrusage, 2, 0},	/* 117  */
  {"getsockopt", getsockopt, 5, 0},	/* 118  */
  {"nosys", 0, 0, 0},			/* 119 resuba (bwa */
  {"readv", readv, 3, 0},		/* 120  */
  {"writev", writev, 2, 0},		/* 121  */
  {"settimeofday", settimeofday, 2, 0},	/* 122  */
  {"fchown", fchown, 3, 0},		/* 123  */
  {"fchmod", fchmod, 2, 0},		/* 124  */
#ifdef COMPAT_43
  {"recvfrom_compat43", recvfrom_compat43, 6, 0}, /* 125  */
#else
  {"nosys", 0, 0, 0},
#endif
#ifdef COMPAT_43
  {"setreuid_compat43", setreuid_compat43, 2, 0}, /* 126  */
#else
  {"nosys", 0, 0, 0},
#endif
#ifdef COMPAT_43
  {"setregid_compat43", setregid_compat43, 2, 0}, /* 127  */
#else
  {"nosys", 0, 0, 0},
#endif
  {"rename", rename, 2, 0},		/* 128  */
  {"truncate", truncate, 2, 0},		/* 129  */
  {"ftruncate", ftruncate, 2, 0},	/* 130  */
  {"flock", flock, 2, 0},		/* 131  */
  {"mkfifo", mkfifo, 2, 0},		/* 132  */
  {"sendto", sendto, 6, 0},		/* 133  */
  {"shutdown", shutdown, 2, 0},		/* 134  */
  {"socketpair", socketpair, 4, 0},	/* 135  */
  {"mkdir", mkdir, 2, 0},		/* 136  */
  {"rmdir", rmdir, 1, 0},		/* 137  */
  {"utimes", utimes, 2, 0},		/* 138  */
  {"nosys", 0, 0, 0},			/* 139 sigreturn weird */
  {"adjtime", adjtime, 2, 0},		/* 140  */
#ifdef COMPAT_43
  {"getpeername_compat43", getpeername_compat43, 3, 0}, /* 141  */
#else
  {"nosys", 0, 0, 0},
#endif
  {"gethostid", gethostid, 0, 0},	/* 142  */
  {"sethostid", sethostid, 1, 0},	/* 143  */
  {"getrlimit", getrlimit, 2, 0},	/* 144  */
  {"setrlimit", setrlimit, 2, 0},	/* 145  */
#ifdef COMPAT_43
  {"killpg_compat43", killpg_compat43, 2, 0}, /* 146  */
#else
  {"nosys", 0, 0, 0},
#endif
  {"setsid", setsid, 0, 0},		/* 147  */
  {"nosys", 0, 0, 0},			/* 148 quotactl blecherousness */
  {"nosys", 0, 0, 0},			/* 149 quota blecherousness */
#ifdef COMPAT_43
  {"getsockname_compat43", getsockname_compat43, 3, 0}, /* 150  */
#else
  {"nosys", 0, 0, 0},
#endif
};

struct sysent sysent_NFS[] = 
{
  {"nosys", 0, 0, 0},			/* 155 nfssvc  */
  {"getdirentries", getdirentries, 4, 0}, /* 156  */
  {"statfs", statfs, 2, 0},		/* 157  */
  {"fstatfs", fstatfs, 2, 0},		/* 158  */
  {"nosys", 0, 0, 0},			/* 159   */
  {"nosys", 0, 0, 0},			/* 160 async_daemon  */
  {"nosys", 0, 0, 0},			/* 161 getfh  */
};

struct sysent sysent_CMU[] = 
{
  {"task2pid", task2pid, 1, 0},		/* -34  */
  {"pid2task", pid2task, 1, 0},		/* -33  */
};

struct sysent sysent_GNU[] = 
{
  {"stat", stat, 2, 0},			/* -200  */
  {"lstat", lstat, 2, 0},		/* -199  */
  {"fstat", fstat, 2, 0},		/* -198  */
  {"getproc", getproc, 0, 0},		/* -197  */
  {"setproc", setproc, 1, 0},		/* -196  */
  {"getauth", getauth, 0, 0},		/* -195  */
  {"setauth", setauth, 1, 0},		/* -194  */
  {"setdtablesize", setdtablesize, 1, 0}, /* -193  */
  {"getdport", getdport, 1, 0},		/* -192  */
  {"portopen", portopen, 1, 0},		/* -191  */
  {"fchcore", fchcore, 1, 0},		/* -190  */
  {"chcore", chcore, 1, 0},		/* -189  */
  {"setcwdir", setcwdir, 1, 0},		/* -188  */
  {"setcrdir", setcrdir, 1, 0},		/* -187  */
  {"setccdir", setccdir, 1, 0},		/* -186  */
  {"getcwdir", getcwdir, 0, 0},		/* -185  */
  {"getcrdir", getcrdir, 0, 0},		/* -184  */
  {"getccdir", getccdir, 0, 0},		/* -183  */
  {"fchroot", fchroot, 1, 0},		/* -182  */
};

struct systab systab[] =
{
  {-200, -182, sysent_GNU},
  {-34, -33, sysent_CMU},
  {155, 161, sysent_NFS},
  {1, 150, sysent_BSD},
  {0, 0, 0},
};
