#include <sys/errno.h>
#define err_posix err_system(0x10)
#define POSIX_EPERM (err_posix|err_sub(0)|EPERM)
#define POSIX_ENOENT (err_posix|err_sub(0)|ENOENT)
#define POSIX_ESRCH (err_posix|err_sub(0)|ESRCH)
#define POSIX_EINTR (err_posix|err_sub(0)|EINTR)
#define POSIX_EIO (err_posix|err_sub(0)|EIO)
#define POSIX_ENXIO (err_posix|err_sub(0)|ENXIO)
#define POSIX_ENOEXEC (err_posix|err_sub(0)|ENOEXEC)
#define POSIX_EBADF (err_posix|err_sub(0)|EBADF)
#define POSIX_ECHILD (err_posix|err_sub(0)|ECHILD)
#define POSIX_EAGAIN (err_posix|err_sub(0)|EAGAIN)
#define POSIX_ENOMEM (err_posix|err_sub(0)|ENOMEM)
#define POSIX_EACCES (err_posix|err_sub(0)|EACCES)
#define POSIX_EFAULT (err_posix|err_sub(0)|EFAULT)
#define POSIX_ENOTBLK (err_posix|err_sub(0)|ENOTBLK)
#define POSIX_EBUSY (err_posix|err_sub(0)|EBUSY)
#define POSIX_EEXIST (err_posix|err_sub(0)|EEXIST)
#define POSIX_EXDEV (err_posix|err_sub(0)|EXDEV)
#define POSIX_ENODEV (err_posix|err_sub(0)|ENODEV)
#define POSIX_ENOTDIR (err_posix|err_sub(0)|ENOTDIR)
#define POSIX_EISDIR (err_posix|err_sub(0)|EISDIR)
#define POSIX_EINVAL (err_posix|err_sub(0)|EINVAL)
#define POSIX_ENFILE (err_posix|err_sub(0)|ENFILE)
#define POSIX_EMFILE (err_posix|err_sub(0)|EMFILE)
#define POSIX_ENOTTY (err_posix|err_sub(0)|ENOTTY)
#define POSIX_ETXTBSY (err_posix|err_sub(0)|ETXTBSY)
#define POSIX_EFBIG (err_posix|err_sub(0)|EFBIG)
#define POSIX_ENOSPC (err_posix|err_sub(0)|ENOSPC)
#define POSIX_ESPIPE (err_posix|err_sub(0)|ESPIPE)
#define POSIX_EROFS (err_posix|err_sub(0)|EROFS)
#define POSIX_EMLINK (err_posix|err_sub(0)|EMLINK)
#define POSIX_EPIPE (err_posix|err_sub(0)|EPIPE)
#define POSIX_EDOM (err_posix|err_sub(0)|EDOM)
#define POSIX_ERANGE (err_posix|err_sub(0)|ERANGE)
#define POSIX_EWOULDBLOCK (err_posix|err_sub(0)|EWOULDBLOCK)
#define POSIX_EINPROGRESS (err_posix|err_sub(0)|EINPROGRESS)
#define POSIX_EALREADY (err_posix|err_sub(0)|EALREADY)
#define POSIX_ENOTSOCK (err_posix|err_sub(0)|ENOTSOCK)
#define POSIX_EDESTADDRREQ (err_posix|err_sub(0)|EDESTADDRREQ)
#define POSIX_EMSGSIZE (err_posix|err_sub(0)|EMSGSIZE)
#define POSIX_EPROTOTYPE (err_posix|err_sub(0)|EPROTOTYPE)
#define POSIX_ENOPROTOOPT (err_posix|err_sub(0)|ENOPROTOOPT)
#define POSIX_EPROTONOSUPPORT (err_posix|err_sub(0)|EPROTONOSUPPORT)
#define POSIX_ESOCKTNOSUPPORT (err_posix|err_sub(0)|ESOCKTNOSUPPORT)
#define POSIX_EOPNOTSUPP (err_posix|err_sub(0)|EOPNOTSUPP)
#define POSIX_EPFNOSUPPORT (err_posix|err_sub(0)|EPFNOSUPPORT)
#define POSIX_EAFNOSUPPORT (err_posix|err_sub(0)|EAFNOSUPPORT)
#define POSIX_EADDRINUSE (err_posix|err_sub(0)|EADDRINUSE)
#define POSIX_EADDRNOTAVAIL (err_posix|err_sub(0)|EADDRNOTAVAIL)
#define POSIX_ENETDOWN (err_posix|err_sub(0)|ENETDOWN)
#define POSIX_ENETUNREACH (err_posix|err_sub(0)|ENETUNREACH)
#define POSIX_ENETRESET (err_posix|err_sub(0)|ENETRESET)
#define POSIX_ECONNABORTED (err_posix|err_sub(0)|ECONNABORTED)
#define POSIX_ECONNRESET (err_posix|err_sub(0)|ECONNRESET)
#define POSIX_ENOBUFS (err_posix|err_sub(0)|ENOBUFS)
#define POSIX_EISCONN (err_posix|err_sub(0)|EISCONN)
#define POSIX_ENOTCONN (err_posix|err_sub(0)|ENOTCONN)
#define POSIX_ESHUTDOWN (err_posix|err_sub(0)|ESHUTDOWN)
#define POSIX_ETOOMANYREFS (err_posix|err_sub(0)|ETOOMANYREFS)
#define POSIX_ETIMEDOUT (err_posix|err_sub(0)|ETIMEDOUT)
#define POSIX_ECONNREFUSED (err_posix|err_sub(0)|ECONNREFUSED)
#define POSIX_ELOOP (err_posix|err_sub(0)|ELOOP)
#define POSIX_ENAMETOOLONG (err_posix|err_sub(0)|ENAMETOOLONG)
#define POSIX_EHOSTDOWN (err_posix|err_sub(0)|EHOSTDOWN)
#define POSIX_EHOSTUNREACH (err_posix|err_sub(0)|EHOSTUNREACH)
#define POSIX_ENOTEMPTY (err_posix|err_sub(0)|ENOTEMPTY)
#define POSIX_EPROCLIM (err_posix|err_sub(0)|EPROCLIM)
#define POSIX_EUSERS (err_posix|err_sub(0)|EUSERS)
#define POSIX_EDQUOT (err_posix|err_sub(0)|EDQUOT)
#define POSIX_ESTALE (err_posix|err_sub(0)|ESTALE)
#define POSIX_EREMOTE (err_posix|err_sub(0)|EREMOTE)
#define POSIX_ENOMSG (err_posix|err_sub(0)|ENOMSG)
#define POSIX_EBADMSG (err_posix|err_sub(0)|EBADMSG)
#define POSIX_EIDRM (err_posix|err_sub(0)|EIDRM)
#define POSIX_EDEADLK (err_posix|err_sub(0)|EDEADLK)
#define POSIX_ENOLCK (err_posix|err_sub(0)|ENOLCK)
