/* This exists because cpp can't deal with mach/mach_traps.h for some
   unknown reason.  */

#pragma once

#include <mach/port.h>

mach_port_t mach_reply_port (void);
mach_port_t mach_thread_self (void);
mach_port_t mach_task_self (void);
mach_port_t mach_host_self (void);

